<?php

namespace FiloBlu\Core\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\OptionSourceInterface;
use Magento\Framework\Mail\EmailMessageInterface;
use Magento\Framework\Mail\MessageInterface;
use Magento\Framework\Mail\TransportInterface;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Registry;
use Magento\Store\Model\ScopeInterface;

/**
 * Class Transport
 * @package FiloBlu\Core\Model
 */
class Transport implements TransportInterface, OptionSourceInterface
{

    /**
     * XML PATH
     */
    const XML_MAIL_TRANSPORT = 'filoblu_mail/mail/transport';
    /**
     * @var MessageInterface
     */
    protected $message;
    /**
     * @var array
     */
    protected $transport;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;
    /**
     * @var Registry
     */
    protected $registry;
    /**
     * @var int|null
     */
    protected $storeId;

    /**
     * Transport constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param ObjectManagerInterface $objectManager
     * @param Registry $registry
     * @param null $message
     * @param array $transport
     * @param int|string|null $storeId
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        ObjectManagerInterface $objectManager,
        Registry $registry,
        $message = null,
        array $transport = [],
        $storeId = null
    )
    {
        $this->transport = $transport;
        $this->scopeConfig = $scopeConfig;

        $this->message = $message;
        $this->objectManager = $objectManager;
        $this->registry = $registry;
        $this->storeId = $storeId;
    }

    /**
     * Send Message
     */
    public function sendMessage()
    {
        $this->objectManager->create(
            $this->transport[$this->getTransport()]['class'],
            ['message' => $this->message, 'storeId' => $this->storeId]
        )->sendMessage();
    }

    /**
     * @return string
     */
    public function getTransport()
    {
        $transport = $this->scopeConfig->getValue(self::XML_MAIL_TRANSPORT, ScopeInterface::SCOPE_STORE, $this->storeId);
        return ($transport != null) ? $transport : 'magento_transport';
    }

    /**
     * @return MessageInterface
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     * @return array
     */
    public function toOptionArray()
    {
        $returnArray = [];
        foreach ($this->transport as $key => $transport) {
            $returnArray[] = ['value' => $key, 'label' => $transport['label']];
        }
        return $returnArray;
    }
}
